#--
# NotCopyable
#
# Adapted from Ruby Treasures 0.4, Copyright (C) 2002 Paul Brannan
#
# Ruby License
#
# This module is free software. You may use, modify, and/or redistribute this
# software under the same terms as Ruby.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.
#
# ==========================================================================
#  Revision History:
# --------------------------------------------------------------------------
#  2004.04  Trans
#    Ported to Mega Modules
# ==========================================================================
#
# TODO: Method Clash
# If one is reopening a class and including this, it will not work
# if the class defined #dub, #clone or #copy (and #deep_copy) itself.
# It only work if it's ancestors are responsible for those methods.
# So how to fix?
#
#++

#:title: NotCopyable
#
# Disable copying via dup and clone (and copy).
#
# == Usage
#
#   require 'mega/notcopyable.rb'
#
#   Class FixedString < String
#     include NotCopyable
#   end
#
#   s = FixedString.new("Don't tred on me!")
#   puts s
#   s2 = s.dup
#
# _produces_
#
#   Don't tred on me!
#   TypeError, "Object not copyable"
#
# == Notes
#
# * I don't think this will prove very useful, but just in case
#   here it is.
#
# == Author(s)
#
# * Paul Brannan <paul at atdesk.com>
# * Thomas Sawyer
#

module NotCopyable

  def clone
    raise TypeError, "Object not copyable"
  end
  def dup
    raise TypeError, "Object not copyable"
  end
  # if object/copy has been required?
  def copy
    raise TypeError, "Object not copyable"
  end
  alias_method( :deep_copy, :copy )
end



#  _____         _
# |_   _|__  ___| |_
#   | |/ _ \/ __| __|
#   | |  __/\__ \ |_
#   |_|\___||___/\__|
#

=begin testing

  require 'test/unit'

  # fixture

  class Foo
    include NotCopyable
  end

  class Bar < Foo
  end

  class Foo2
  end

  # test

  class TC_NotCopyable < Test::Unit::TestCase

    def test_dup
      f = Foo.new
      assert_raises(TypeError) do
        g = f.dup
      end
    end

    def test_dup_derived
      f = Bar.new
      assert_raises(TypeError) do
        g = f.dup
      end
    end

    def test_dup_okay
      f = Foo2.new
      g = f.dup
    end

    def test_clone
      f = Foo.new
      assert_raises(TypeError) do
        g = f.clone
      end
    end

    def test_clone_derived
      f = Bar.new
      assert_raises(TypeError) do
        g = f.clone
      end
    end

    def test_clone_okay
      f = Foo2.new
      g = f.clone
    end

  end

=end
